@@ -7,30 +7,23 @@ Credits:
 - https://gist.github.com/fnky/458719343aabd01cfb17a3a4f7296797
 """
 
+# The entirety of Terminal will soon be moved over to a new submodule, so
+# this ignore is temporary.
+# pylint: disable=too-many-lines
+
 from __future__ import annotations
 
 import re
-import sys
-import signal
 
-from typing import Optional, Any, Union, Callable, Tuple, Pattern
+from typing import Optional, Any, Union, Pattern
 from dataclasses import dataclass, fields
 from enum import Enum, auto as _auto
-from sys import stdout as _stdout
-from string import hexdigits
 from os import name as _name, system
-from shutil import get_terminal_size
 
 from .input import getch
-
-_IS_NT = _name == "nt"
+from .terminal import terminal
 
 __all__ = [
-    "Color",
-    "foreground",
-    "background",
-    "terminal",
-    "is_interactive",
     "save_screen",
     "restore_screen",
     "set_alt_buffer",
@@ -77,258 +70,6 @@ RE_MOUSE: dict[str, Pattern] = {
 }
 
 
-class Color:
-    """Class to store various color utilities
-
-    Two instances of this class are provided, `foreground`
-    and `background`. The difference between these is the color
-    layer they operate on.
-
-    To use this class you should call either instances with some
-    data type representing a color. The following patterns are supported:
-
-    - `int`: 0-256 terminal colors
-    - `str`: Name of one of the registered named colors. See `Color.names`.
-    - `#rrggbb`: RGB hex string. Note: alpha values are not supported.
-    - `tuple[int, int]`: Tuple of RGB colors, each 0-256.
-
-    When calling an instance of the class, the following args are set:
-
-    Args:
-        text: The string that will be colored.
-        color: The color specifier.
-        reset_color: A boolean that determines whether the returned value should
-            end with a reset code. Defaults to True.
-
-    Returns:
-        The colored string.
-
-    Raises:
-        NotImplementedError: Unrecognized color specifier was passed.
-    """
-
-    ColorType = Union[int, str, Tuple[int, int, int]]
-    """A simple type to represent color patterns. See `Color` for more info."""
-
-    names = {
-        "black": 0,
-        "red": 1,
-        "green": 2,
-        "yellow": 3,
-        "blue": 4,
-        "magenta": 5,
-        "cyan": 6,
-        "white": 7,
-        "grey": 8,
-        "brightred": 9,
-        "brightgreen": 10,
-        "brightyellow": 11,
-        "brightblue": 12,
-        "brightmagenta": 13,
-        "brightcyan": 14,
-        "brightwhite": 15,
-    }
-    """16 default named colors. Expanding this list will expand the names `pytermgui.parser.markup`
-    will recognize, but if that is your objective it is better to use
-    `pytermgui.parser.MarkupLanguage.alias`."""
-
-    def __init__(self, layer: int = 0) -> None:
-        """Initialize Color object.
-
-        Args:
-            layer: One of [0, 1]. Objects instantiated with 0 will represent foreground
-                colors, while objects instantiated with 1 will display background ones.
-
-        Raises:
-            NotImplementedError: Invalid layer.
-        """
-
-        if layer not in [0, 1]:
-            raise NotImplementedError(f"Layer {layer} can only be one of [0, 1].")
-
-        self.layer_offset = layer * 10
-
-    @staticmethod
-    def translate_hex(color: str) -> tuple[int, int, int]:
-        """Translates a hex string of format #RRGGBB into an RGB tuple of integers.
-
-        Args:
-            color: The hexadecimal string to convert. Must follow the format #RRGGBB, where
-                capitalization is optional.
-
-        Returns:
-            A tuple of integers, 0-255 each, standing for red, green and blue values in that
-            order.
-        """
-
-        if color.startswith("#"):
-            color = color[1:]
-
-        rgb = []
-        for i in (0, 2, 4):
-            color_hex = color[i : i + 2]
-            rgb.append(int(color_hex, 16))
-
-        return rgb[0], rgb[1], rgb[2]
-
-    def __call__(self, text: str, color: ColorType, reset_color: bool = True) -> str:
-        """Colors a piece of text. See help(Color) for more info."""
-
-        # convert hex string to tuple[int, int, int]
-        if isinstance(color, str) and all(
-            char in hexdigits or char == "#" for char in color
-        ):
-            try:
-                color = self.translate_hex(color)
-            except ValueError:
-                # value is not a hex number, but is string
-                pass
-
-        if color in self.names:
-            new = self.names[str(color)]
-            assert isinstance(new, int)
-            color = new
-
-        # rgb values
-        if isinstance(color, tuple):
-            red, green, blue = color
-            color_value = f"2;{red};{green};{blue}m"
-
-        # 8-bit colors (including 0-16)
-        elif isinstance(color, int) or color.isdigit():
-            color_value = f"5;{color}m"
-
-        else:
-            raise NotImplementedError(
-                f"Not sure what to do with {color} of type {type(color)}"
-            )
-
-        return (
-            f"\x1b[{38 + self.layer_offset};"
-            + color_value
-            + text
-            + (set_mode("reset") if reset_color else "")
-        )
-
-
-foreground = Color()
-"""`Color` instance for setting foreground colors."""
-
-background = Color(1)
-"""`Color` instance for setting background colors."""
-
-
-class Terminal:
-    """A class to store & access data about a terminal."""
-
-    RESIZE = 0
-    """Event sent out when the terminal has been resized.
-
-    Arguments passed:
-    - New size: tuple[int, int]
-    """
-
-    margins = [0, 0, 0, 0]
-    """Not quite sure what this does at the moment."""
-
-    def __init__(self) -> None:
-        """Initialize `_Terminal` class."""
-
-        self.origin: tuple[int, int] = (1, 1)
-        self.size: tuple[int, int] = self._get_size()
-        self._listeners: dict[int, list[Callable[..., Any]]] = {}
-
-        if hasattr(signal, "SIGWINCH"):
-            signal.signal(signal.SIGWINCH, self._update_size)
-
-        # TODO: Support SIGWINCH on Windows.
-
-    def _call_listener(self, event: int, data: Any) -> None:
-        """Calls callbacks for event.
-
-        Args:
-            event: A terminal event.
-            data: Arbitrary data passed to the callback.
-        """
-
-        if event in self._listeners:
-            for callback in self._listeners[event]:
-                callback(data)
-
-    def _get_size(self) -> tuple[int, int]:
-        """Gets the screen size with origin substracted."""
-
-        size = get_terminal_size()
-        return (size[0] - self.origin[0], size[1] - self.origin[1])
-
-    def _update_size(self, *_: Any) -> None:
-        """Resize terminal when SIGWINCH occurs, and call listeners."""
-
-        self.size = self._get_size()
-        self._call_listener(self.RESIZE, self.size)
-
-    @property
-    def width(self) -> int:
-        """Gets the current width of the terminal."""
-
-        return self.size[0]
-
-    @property
-    def height(self) -> int:
-        """Gets the current height of the terminal."""
-
-        return self.size[1]
-
-    def subscribe(self, event: int, callback: Callable[..., Any]) -> None:
-        """Subcribes a callback to be called when event occurs.
-
-        Args:
-            event: The terminal event that calls callback.
-            callback: The callable to be called. The signature of this
-                callable is dependent on the event. See the documentation
-                of the specific event for more information.
-        """
-
-        if not event in self._listeners:
-            self._listeners[event] = []
-
-        self._listeners[event].append(callback)
-
-    def fill(self, color: int = 0, flush: bool = True) -> None:
-        """Fills the entire terminal with the given color.
-
-        This is more of a debug function than anything, as performance is not the
-        greatest.
-
-        Args:
-            color: An integer for the color to fill in. Must be in the 0-255 xterm
-                color range.
-            flush: Whether stdout should be flushed. Defaults to True, but setting
-                it False can be beneficial for performance, as long as you flush in
-                some other place.
-        """
-
-        for height in range(self.height):
-            sys.stdout.write(
-                f"\033[{height};0H"
-                + background(" " * (self.width - 1), color)
-                + ("\n" if flush else "")
-            )
-
-
-terminal = Terminal()
-"""Terminal instance that should be used pretty much always."""
-
-
-def is_interactive() -> bool:
-    """Determines whether shell is interactive.
-
-    A shell is interactive if it is run from `python3` or `python3 -i`.
-    """
-
-    return hasattr(sys, "ps1")
-
-
 # screen commands
 def save_screen() -> None:
     """Saves the contents of the screen, and wipes it.
@@ -381,7 +122,7 @@ def clear(what: str = "screen") -> None:
         "line": "\x1b[2K",
     }
 
-    _stdout.write(commands[what])
+    terminal.write(commands[what])
 
 
 # cursor commands
@@ -403,13 +144,13 @@ def save_cursor() -> None:
     Use `restore_cursor()` to restore it.
     """
 
-    _stdout.write("\x1b[s")
+    terminal.write("\x1b[s")
 
 
 def restore_cursor() -> None:
     """Restore cursor position as saved by `save_cursor`."""
 
-    _stdout.write("\x1b[u")
+    terminal.write("\x1b[u")
 
 
 def report_cursor() -> tuple[int, int] | None:
@@ -445,7 +186,7 @@ def move_cursor(pos: tuple[int, int]) -> None:
     """
 
     posx, posy = pos
-    _stdout.write(f"\x1b[{posy};{posx}H")
+    terminal.write(f"\x1b[{posy};{posx}H")
 
 
 def cursor_up(num: int = 1) -> None:
@@ -459,7 +200,7 @@ def cursor_up(num: int = 1) -> None:
         can do it manually with `sys.stdout.flush()`.
     """
 
-    _stdout.write(f"\x1b[{num}A")
+    terminal.write(f"\x1b[{num}A")
 
 
 def cursor_down(num: int = 1) -> None:
@@ -473,7 +214,7 @@ def cursor_down(num: int = 1) -> None:
         can do it manually with `sys.stdout.flush()`.
     """
 
-    _stdout.write(f"\x1b[{num}B")
+    terminal.write(f"\x1b[{num}B")
 
 
 def cursor_right(num: int = 1) -> None:
@@ -487,7 +228,7 @@ def cursor_right(num: int = 1) -> None:
         can do it manually with `sys.stdout.flush()`.
     """
 
-    _stdout.write(f"\x1b[{num}C")
+    terminal.write(f"\x1b[{num}C")
 
 
 def cursor_left(num: int = 1) -> None:
@@ -501,7 +242,7 @@ def cursor_left(num: int = 1) -> None:
         can do it manually with `sys.stdout.flush()`.
     """
 
-    _stdout.write(f"\x1b[{num}D")
+    terminal.write(f"\x1b[{num}D")
 
 
 def cursor_next_line(num: int = 1) -> None:
@@ -515,7 +256,7 @@ def cursor_next_line(num: int = 1) -> None:
         can do it manually with `sys.stdout.flush()`.
     """
 
-    _stdout.write(f"\x1b[{num}E")
+    terminal.write(f"\x1b[{num}E")
 
 
 def cursor_prev_line(num: int = 1) -> None:
@@ -529,7 +270,7 @@ def cursor_prev_line(num: int = 1) -> None:
         can do it manually with `sys.stdout.flush()`.
     """
 
-    _stdout.write(f"\x1b[{num}F")
+    terminal.write(f"\x1b[{num}F")
 
 
 def cursor_column(num: int = 0) -> None:
@@ -543,7 +284,7 @@ def cursor_column(num: int = 0) -> None:
         can do it manually with `sys.stdout.flush()`.
     """
 
-    _stdout.write(f"\x1b[{num}G")
+    terminal.write(f"\x1b[{num}G")
 
 
 def cursor_home() -> None:
@@ -554,7 +295,7 @@ def cursor_home() -> None:
         can do it manually with `sys.stdout.flush()`.
     """
 
-    _stdout.write("\x1b[H")
+    terminal.write("\x1b[H")
 
 
 def set_mode(mode: Union[str, int], write: bool = True) -> str:
@@ -602,7 +343,7 @@ def set_mode(mode: Union[str, int], write: bool = True) -> str:
 
     code = f"\x1b[{mode}m"
     if write:
-        _stdout.write(code)
+        terminal.write(code)
 
     return code
 
@@ -657,6 +398,8 @@ class MouseAction(Enum):
     HOVER = _auto()
     """Mouse moved without clicking."""
 
+    # TODO: Support left & right mouse release separately, without breaking
+    #       current API.
     RELEASE = _auto()
     """Mouse button released; end of any and all mouse action sequences."""
 
@@ -703,6 +446,21 @@ class MouseEvent:
 
         return self
 
+    def is_scroll(self) -> bool:
+        """Returns True if event.action is one of the scrolling actions."""
+
+        return self.action in {MouseAction.SCROLL_DOWN, MouseAction.SCROLL_UP}
+
+    def is_primary(self) -> bool:
+        """Returns True if event.action is one of the primary (left-button) actions."""
+
+        return self.action in {MouseAction.LEFT_CLICK, MouseAction.LEFT_DRAG}
+
+    def is_secondary(self) -> bool:
+        """Returns True if event.action is one of the secondary (secondary-button) actions."""
+
+        return self.action in {MouseAction.RIGHT_CLICK, MouseAction.RIGHT_DRAG}
+
 
 def report_mouse(
     event: str, method: Optional[str] = "decimal_xterm", stop: bool = False
@@ -742,30 +500,30 @@ def report_mouse(
     """
 
     if event == "press":
-        _stdout.write("\x1b[?1000")
+        terminal.write("\x1b[?1000")
 
     elif event == "highlight":
-        _stdout.write("\x1b[?1001")
+        terminal.write("\x1b[?1001")
 
     elif event == "press_hold":
-        _stdout.write("\x1b[?1002")
+        terminal.write("\x1b[?1002")
 
     elif event == "hover":
-        _stdout.write("\x1b[?1003")
+        terminal.write("\x1b[?1003")
 
     else:
         raise NotImplementedError(f"Mouse report event {event} is not supported!")
 
-    _stdout.write("l" if stop else "h")
+    terminal.write("l" if stop else "h")
 
     if method == "decimal_utf8":
-        _stdout.write("\x1b[?1005")
+        terminal.write("\x1b[?1005")
 
     elif method == "decimal_xterm":
-        _stdout.write("\x1b[?1006")
+        terminal.write("\x1b[?1006")
 
     elif method == "decimal_urxvt":
-        _stdout.write("\x1b[?1015")
+        terminal.write("\x1b[?1015")
 
     elif method is None:
         return
@@ -773,8 +531,7 @@ def report_mouse(
     else:
         raise NotImplementedError(f"Mouse report method {method} is not supported!")
 
-    _stdout.write("l" if stop else "h")
-    _stdout.flush()
+    terminal.write("l" if stop else "h", flush=True)
 
 
 def translate_mouse(code: str, method: str) -> list[MouseEvent | None] | None:
@@ -791,11 +548,15 @@ def translate_mouse(code: str, method: str) -> list[MouseEvent | None] | None:
         and no codes could be determined None is returned.
     """
 
+    if code == "\x1b":
+        return None
+
     mouse_codes = {
         "decimal_xterm": {
             "0M": MouseAction.LEFT_CLICK,
             "0m": MouseAction.RELEASE,
-            "2": MouseAction.RIGHT_CLICK,
+            "2M": MouseAction.RIGHT_CLICK,
+            "2m": MouseAction.RELEASE,
             "32": MouseAction.LEFT_DRAG,
             "34": MouseAction.RIGHT_DRAG,
             "35": MouseAction.HOVER,
@@ -817,6 +578,7 @@ def translate_mouse(code: str, method: str) -> list[MouseEvent | None] | None:
     pattern: Pattern = RE_MOUSE[method]
 
     events: list[MouseEvent | None] = []
+
     for sequence in code.split("\x1b"):
         if len(sequence) == 0:
             continue
@@ -830,7 +592,7 @@ def translate_mouse(code: str, method: str) -> list[MouseEvent | None] | None:
 
             # decimal_xterm uses the last character's
             # capitalization to signify press/release state
-            if len(release_code) > 0 and identifier == "0":
+            if len(release_code) > 0 and identifier in ["0", "2"]:
                 identifier += release_code
 
             if identifier in mapping:
@@ -859,13 +621,7 @@ def print_to(pos: tuple[int, int], *args: Any, **kwargs: Any) -> None:
 
 
 def reset() -> str:
-    """Resets printing mode.
-
-    Args:
-        reset_style: Boolean that determines whether a reset
-            character should be appended to the end of the
-            string.
-    """
+    """Resets printing mode."""
 
     return set_mode("reset", False)
 
