@@ -13,7 +13,6 @@ from contextlib import contextmanager
 from typing import Callable, Generator, Any, Union, List
 
 from .ansi_interface import (
-    is_interactive,
     save_cursor,
     restore_cursor,
     print_to,
@@ -29,6 +28,12 @@ from .ansi_interface import (
     MouseEvent,
 )
 
+from .terminal import get_terminal
+
+# This is technically meant to be here, but it has to be in `input.py` due
+# to some package structure issues.
+from .input import timeout  # pylint: disable=unused-import
+
 # TODO: Move this absolute beast to a types submodule
 MouseTranslator = Callable[[str], Union[List[Union[MouseEvent, None]], None]]
 
@@ -76,10 +81,12 @@ def alt_buffer(echo: bool = False, cursor: bool = True) -> Generator[None, None,
         cursor: Whether `hide_cursor` should be called on startup.
     """
 
+    terminal = get_terminal()
+
     try:
         set_alt_buffer()
 
-        if not echo and name == "posix" and not is_interactive():
+        if not echo and name == "posix" and not terminal.is_interactive():
             unset_echo()
 
         if not cursor:
@@ -90,7 +97,7 @@ def alt_buffer(echo: bool = False, cursor: bool = True) -> Generator[None, None,
     finally:
         unset_alt_buffer()
 
-        if not echo and name == "posix" and not is_interactive():
+        if not echo and name == "posix" and not terminal.is_interactive():
             set_echo()
             cursor_up()
 
