@@ -14,6 +14,10 @@ __all__ = [
 ]
 
 
+class TimeoutException(Exception):
+    """Raised when an action has timed out."""
+
+
 class WidthExceededError(Exception):
     """Raised when an element's width is larger than the screen."""
 
@@ -22,6 +26,10 @@ class LineLengthError(Exception):
     """Raised when a widget line is not the expected length."""
 
 
+class ColorSyntaxError(Exception):
+    """Raised when a color string could not be parsed into a `pytermgui.colors.Color`"""
+
+
 @dataclass
 class ParserSyntaxError(Exception):
     """Parent exception for unparsable strings.
@@ -40,12 +48,16 @@ class ParserSyntaxError(Exception):
     def message(self) -> str:
         """Create message from tag, context and cause."""
 
-        escaped_context = ascii(self.context).strip("'")
         start, end = self._delimiters
-        return (
-            f'Tag "{start}{self.tag}{end}" in string "{escaped_context}" {self.cause}.'
+        full = start + self.tag + end
+
+        escaped_context = ascii(self.context).strip("'")
+        highlighted = escaped_context.replace(
+            full, "\x1b[31m\x1b[1m" + full + "\x1b[0m", 1
         )
 
+        return f'Tag "{start}{self.tag}{end}" in string "{highlighted}" {self.cause}.'
+
     def escape_message(self) -> str:
         """Return message with markup tags escaped."""
 
