@@ -107,8 +107,7 @@ widgets:
 
 import pytermgui as ptg
 
-loader = ptg.YamlLoader()
-with open("data.yaml", "r") as datafile:
+with ptg.YamlLoader() as loader, open("data.yaml", "r") as datafile:
     namespace = loader.load(datafile)
 
 with ptg.WindowManager() as manager:
@@ -129,7 +128,7 @@ from abc import abstractmethod, ABC
 
 import json
 
-from . import widgets
+from . import widgets as widgets_m
 from .parser import markup
 from .serializer import Serializer
 
@@ -155,10 +154,10 @@ class WidgetNamespace:
     # but not in the code below. It only seems to happen
     # in certain pylint configs as well.
     config: dict[
-        Type[widgets.Widget], dict[str, Any]  # pylint: disable=undefined-variable
+        Type[widgets_m.Widget], dict[str, Any]  # pylint: disable=undefined-variable
     ]
-    widgets: dict[str, widgets.Widget]
-    boxes: dict[str, widgets.boxes.Box] = field(default_factory=dict)
+    widgets: dict[str, widgets_m.Widget]
+    boxes: dict[str, widgets_m.boxes.Box] = field(default_factory=dict)
 
     @classmethod
     def from_config(cls, data: dict[Any, Any], loader: FileLoader) -> WidgetNamespace:
@@ -179,12 +178,12 @@ class WidgetNamespace:
                 raise KeyError(f"Unknown widget type {name}.")
 
             namespace.config[obj] = {
-                "styles": obj.styles.copy(),
+                "styles": obj.styles,
                 "chars": obj.chars.copy(),
             }
 
             for category, inner in config.items():
-                value: str | widgets.styles.MarkupFormatter
+                value: str | widgets_m.styles.MarkupFormatter
 
                 if category not in namespace.config[obj]:
                     setattr(obj, category, inner)
@@ -198,19 +197,18 @@ class WidgetNamespace:
 
     @staticmethod
     def _apply_section(
-        widget: Type[widgets.Widget], title: str, section: dict[str, str]
+        widget: Type[widgets_m.Widget], title: str, section: dict[str, str]
     ) -> None:
         """Applies configuration section to the widget."""
 
         for key, value in section.items():
             if title == "styles":
-                if isinstance(value, str):
-                    widget.set_style(key, widgets.styles.MarkupFormatter(value))
+                widget.set_style(key, value)
                 continue
 
             widget.set_char(key, value)
 
-    def apply_to(self, widget: widgets.Widget) -> None:
+    def apply_to(self, widget: widgets_m.Widget) -> None:
         """Applies namespace config to the widget.
 
         Args:
@@ -218,7 +216,7 @@ class WidgetNamespace:
         """
 
         def _apply_sections(
-            data: dict[str, dict[str, str]], widget: widgets.Widget
+            data: dict[str, dict[str, str]], widget: widgets_m.Widget
         ) -> None:
             """Applies sections from data to the widget."""
 
@@ -247,7 +245,7 @@ class WidgetNamespace:
             for title, section in settings.items():
                 self._apply_section(widget, title, section)
 
-    def __getattr__(self, attr: str) -> widgets.Widget:
+    def __getattr__(self, attr: str) -> widgets_m.Widget:
         """Get widget by name from widget list."""
 
         if attr in self.widgets:
@@ -288,7 +286,18 @@ class FileLoader(ABC):
 
         self.serializer = serializer
 
-    def register(self, cls: Type[widgets.Widget]) -> None:
+    def __enter__(self) -> FileLoader:
+        """Starts context manager."""
+
+        return self
+
+    def __exit__(self, _: Any, exception: Exception, __: Any) -> bool:
+        """Ends context manager."""
+
+        if exception is not None:
+            raise exception
+
+    def register(self, cls: Type[widgets_m.Widget]) -> None:
         """Registers a widget to the serializer.
 
         Args:
@@ -336,7 +345,7 @@ class FileLoader(ABC):
 
         # Create boxes
         for name, inner in (parsed.get("boxes") or {}).items():
-            self.serializer.register_box(name, widgets.boxes.Box(inner))
+            self.serializer.register_box(name, widgets_m.boxes.Box(inner))
 
         # Create widgets
         for name, inner in (parsed.get("widgets") or {}).items():
