@@ -64,8 +64,11 @@ from . import boxes
 
 from .base import *
 from .styles import *
-from .layouts import *
+from .containers import *
 from .interactive import *
+from .collapsible import *
+from .pixel_matrix import *
+from .color_picker import ColorPicker
 
 WidgetType = Union[Widget, Type[Widget]]
 
