@@ -10,13 +10,15 @@ from __future__ import annotations
 
 from copy import deepcopy
 from inspect import signature
-from typing import Callable, Optional, Type, Iterator, Any, Union
+from typing import Callable, Optional, Type, Iterator, Any, Union, Generator
 
 from ..input import keys
 from ..parser import markup
-from ..helpers import real_length, break_line
-from ..ansi_interface import terminal, MouseEvent
-from ..enums import SizePolicy, HorizontalAlignment
+from ..regex import real_length
+from ..helpers import break_line
+from ..terminal import get_terminal, Terminal
+from ..ansi_interface import MouseEvent, reset
+from ..enums import SizePolicy, HorizontalAlignment, WidgetChange
 
 from . import styles as w_styles
 
@@ -40,16 +42,9 @@ def _set_obj_or_cls_style(
         Type[Widget] | Widget: The updated class.
 
     Raises:
-        KeyError: The style key provided is invalid.
-        ValueError: The style value is not callable.
+        See `pytermgui.widgets.styles.StyleManager`.
     """
 
-    if not key in obj_or_cls.styles.keys():
-        raise KeyError(f"Style {key} is not valid for {obj_or_cls}!")
-
-    if not callable(value):
-        raise ValueError(f"Style {key} for {type(obj_or_cls)} has to be a callable.")
-
     obj_or_cls.styles[key] = value
 
     return obj_or_cls
@@ -80,13 +75,13 @@ def _set_obj_or_cls_char(
     return obj_or_cls
 
 
-class Widget:
+class Widget:  # pylint: disable=too-many-public-methods
     """The base of the Widget system"""
 
     set_style = classmethod(_set_obj_or_cls_style)
     set_char = classmethod(_set_obj_or_cls_char)
 
-    styles: dict[str, w_styles.StyleType] = {}
+    styles = w_styles.StyleManager()
     """Default styles for this class"""
 
     chars: dict[str, w_styles.CharType] = {}
@@ -132,11 +127,11 @@ class Widget:
 
         self.width = 1
         self.height = 1
-        self.pos = terminal.origin
+        self.pos = self.terminal.origin
 
         self.depth = 0
 
-        self.styles = type(self).styles.copy()
+        self.styles = type(self).styles.branch(self)
         self.chars = type(self).chars.copy()
 
         self.parent: Widget | None = None
@@ -147,6 +142,7 @@ class Widget:
         self._serialized_fields = type(self).serialized
         self._bindings: dict[str | Type[MouseEvent], tuple[BoundCallback, str]] = {}
         self._relative_width: float | None = None
+        self._previous_state: tuple[tuple[int, int], list[str]] | None = None
 
         for attr, value in attrs.items():
             setattr(self, attr, value)
@@ -160,6 +156,16 @@ class Widget:
 
         return self.debug()
 
+    def __fancy_repr__(self) -> Generator[str, None, None]:
+        """Yields the repr of this object, then a preview of it."""
+
+        yield self.debug()
+        yield "\n\n"
+        yield {
+            "text": "\n".join((line + reset() for line in self.get_lines())),
+            "highlight": False,
+        }
+
     def __iter__(self) -> Iterator[Widget]:
         """Return self for iteration"""
 
@@ -292,6 +298,40 @@ class Widget:
         self.size_policy = SizePolicy.RELATIVE
         self._relative_width = min(1.0, value)
 
+    @property
+    def terminal(self) -> Terminal:
+        """Returns the current global terminal instance."""
+
+        return get_terminal()
+
+    def get_change(self) -> WidgetChange | None:
+        """Determines whether widget lines changed since the last call to this function."""
+
+        lines = self.get_lines()
+
+        if self._previous_state is None:
+            self._previous_state = (self.width, self.height), lines
+            return WidgetChange.LINES
+
+        lines = self.get_lines()
+        (old_width, old_height), old_lines = self._previous_state
+
+        self._previous_state = (self.width, self.height), lines
+
+        if old_width != self.width and old_height != self.height:
+            return WidgetChange.SIZE
+
+        if old_width != self.width:
+            return WidgetChange.WIDTH
+
+        if old_height != self.height:
+            return WidgetChange.HEIGHT
+
+        if old_lines != lines:
+            return WidgetChange.LINES
+
+        return None
+
     def contains(self, pos: tuple[int, int]) -> bool:
         """Determines whether widget contains `pos`.
 
@@ -408,8 +448,10 @@ class Widget:
     def _get_style(self, key: str) -> w_styles.DepthlessStyleType:
         """Gets style call from its key.
 
+        This is analogous to using `self.styles.{key}`
+
         Args:
-            key: A key into the widget's styles dictionary.
+            key: A key into the widget's style manager.
 
         Returns:
             A `pytermgui.styles.StyleCall` object containing the referenced
@@ -420,9 +462,7 @@ class Widget:
             KeyError: Style key is invalid.
         """
 
-        style_method = self.styles[key]
-
-        return w_styles.StyleCall(self, style_method)
+        return self.styles[key]
 
     def _get_char(self, key: str) -> w_styles.CharType:
         """Gets character from its key.
@@ -494,6 +534,11 @@ class Widget:
 
         self._bindings[key] = (action, description)
 
+    def unbind(self, key: str) -> None:
+        """Unbinds the given key."""
+
+        del self._bindings[key]
+
     def execute_binding(self, key: Any) -> bool:
         """Executes a binding belonging to key, when present.
 
@@ -618,23 +663,116 @@ class Label(Widget):
     </p>
     """
 
-    styles: dict[str, w_styles.StyleType] = {"value": w_styles.MARKUP}
-
     serialized = Widget.serialized + ["*value", "align", "padding"]
+    styles = w_styles.StyleManager(value=w_styles.MARKUP)
 
-    def __init__(self, value: str = "", padding: int = 0, **attrs: Any) -> None:
-        """Set up object"""
+    def __init__(
+        self,
+        value: str = "",
+        style: str | w_styles.StyleValue = "",
+        padding: int = 0,
+        non_first_padding: int = 0,
+        **attrs: Any,
+    ) -> None:
+        """Initializes a Label.
+
+        Args:
+            value: The value of this string. Using the default value style
+                (`pytermgui.widgets.styles.MARKUP`),
+            style: A pre-set value for self.styles.value.
+            padding: The number of space (" ") characters to prepend to every line after
+                line breaking.
+            non_first_padding: The number of space characters to prepend to every
+                non-first line of `get_lines`. This is applied on top of `padding`.
+        """
 
         super().__init__(**attrs)
 
         self.value = value
         self.padding = padding
+        self.non_first_padding = non_first_padding
         self.width = real_length(value) + self.padding
 
+        if style != "":
+            self.styles.value = style
+
     def get_lines(self) -> list[str]:
         """Get lines representing this Label, breaking lines as necessary"""
 
-        value_style = self._get_style("value")
-        line_gen = break_line(value_style(self.padding * " " + self.value), self.width)
+        lines = []
+        limit = self.width - self.padding
+        broken = break_line(
+            self.styles.value(self.value),
+            limit=limit,
+            non_first_limit=limit - self.non_first_padding,
+        )
 
-        return list(line_gen) or [""]
+        for i, line in enumerate(broken):
+            if i == 0:
+                lines.append(self.padding * " " + line)
+                continue
+
+            lines.append(self.padding * " " + self.non_first_padding * " " + line)
+
+        return lines or [""]
+
+
+class ScrollableWidget(Widget):
+    """A widget with some scrolling helper methods.
+
+    This is not an implementation of the scrolling behaviour itself, just the
+    user-facing API for it.
+
+    It provides a `_scroll_offset` attribute, which is an integer describing the current
+    scroll state offset from the top, as well as some methods to modify the state."""
+
+    def __init__(self, **attrs: Any) -> None:
+        """Initializes the scrollable widget."""
+
+        super().__init__(**attrs)
+
+        self._max_scroll = 0
+        self._scroll_offset = 0
+
+    def scroll(self, offset: int) -> bool:
+        """Scrolls to given offset, returns the new scroll_offset.
+
+        Args:
+            offset: The amount to scroll by. Positive offsets scroll down,
+                negative up.
+
+        Returns:
+            True if the scroll offset changed, False otherwise.
+        """
+
+        base = self._scroll_offset
+
+        self._scroll_offset = min(
+            max(0, self._scroll_offset + offset), self._max_scroll
+        )
+
+        return base != self._scroll_offset
+
+    def scroll_end(self, end: int) -> int:
+        """Scrolls to either top or bottom end of this object.
+
+        Args:
+            end: The offset to scroll to. 0 goes to the very top, -1 to the
+                very bottom.
+
+        Returns:
+            True if the scroll offset changed, False otherwise.
+        """
+
+        base = self._scroll_offset
+
+        if end == 0:
+            self._scroll_offset = 0
+
+        elif end == -1:
+            self._scroll_offset = self._max_scroll
+
+        return base != self._scroll_offset
+
+    def get_lines(self) -> list[str]:
+        ...
